/*==================================================================================================

    Module Name:  bmp.c

    General Description: IPU test BMP decoder.

====================================================================================================
Copyright (c) 2005, 2007 Motorola, Inc.

Revision History:
 Date         Author      Comment
 ===========  ==========  ==================================================
 14-Sep-2005  Motorola    Initial revision.
 03-Jul-2007  Motorola    Compliance issue fix
 24-Jul-2007  Motorola    Compliance issue fix     
 11-Aug-2007  Motorola    Complicace issue fix

Portability: Indicate if this module is portable to other compilers or
  platforms. If not, indicate specific reasons why is it not portable.


====================================================================================================
                                        INCLUDE FILES
==================================================================================================*/
#include <string.h>
#include <stdio.h>

/*==================================================================================================
                                 GLOBAL FUNCTION PROTOTYPES
==================================================================================================*/

/*==================================================================================================
                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/
typedef struct
{
    unsigned int file_size_bytes;
    unsigned int reserved;
    unsigned int img_offset_bytes;
}   BMP_HDR_T;

typedef struct
{
    unsigned int hdr_size_bytes;
    unsigned int img_width;
    unsigned int img_height;
    unsigned short num_color_planes;
    unsigned short bits_per_pixel;
    unsigned int compression_type;
    unsigned int num_img_bytes;
    unsigned int x_res, y_res;
    unsigned int num_colors;
    unsigned int imp_colors;
}   BMP_INFO_T;

/*==================================================================================================
                                     LOCAL CONSTANTS
==================================================================================================*/
/*==================================================================================================
                                      LOCAL VARIABLES
==================================================================================================*/
/*==================================================================================================
                                     LOCAL FUNCTIONS
==================================================================================================*/
#ifdef BIG_E
static unsigned int endian_uint(unsigned int le_val)
{
    unsigned int be_val;
    unsigned char *pIn, *pOut;

    if(le_val == 0)
    {
        return le_val;
    }

    pIn = (unsigned char *)&le_val;
    pOut = (unsigned char *)&be_val;

    pOut[3] = pIn[0];
    pOut[2] = pIn[1];
    pOut[1] = pIn[2];
    pOut[0] = pIn[3];

    return be_val;
}

/*================================================================================================*/

  static unsigned short endian_ushort(unsigned short le_val)
{
    unsigned short be_val;
    unsigned char *pIn, *pOut;

    if(le_val == 0)
    {
        return le_val;
    }

    pIn = (unsigned char *)&le_val;
    pOut = (unsigned char *)&be_val;

    pOut[1] = pIn[0];
    pOut[0] = pIn[1];

    return be_val;
}
#else
#define endian_ushort(x) x
#define endian_uint(x) x
#endif
/*==================================================================================================
                                       GLOBAL FUNCTIONS
==================================================================================================*/
int 
bmp_decode
(
    unsigned char *pBMPData,
    unsigned char *pBMPImg,
    unsigned int *fh,
    unsigned int *fw
)
{
    unsigned char *pBMP, *pUC;
//    unsigned short *pShort;
    unsigned int i;

    BMP_HDR_T * pHdr;
    BMP_INFO_T * pInfo;

    pBMP = pBMPData;
    if(memcmp(pBMP, "BM", 2))
    {
        return(-1);
    }

    pBMP += sizeof(short);

    pHdr = (BMP_HDR_T *)pBMP;
    pBMP += sizeof(BMP_HDR_T);

    pInfo = (BMP_INFO_T *)pBMP;
    pBMP += sizeof(BMP_INFO_T);

#ifdef NOT_NOW
    pShort = (unsigned short *)pInfo;
    pShort += 7;

    if(endian_ushort(*pShort) != 24) // Must be a 24-bit image
    {
printf("*pShort=0x%04X *(pShort-1)=0x%04X *(pShort+1)=0x%04X\n", *pShort, *(pShort-1), *(pShort+1));
        return(-2);
    }

    pShort++;

    if(endian_ushort(*pShort) != 0) // must be a non-compressed image
    {
        return(-3);
    }

    pShort++;

    if(endian_ushort(*pShort) != 0) // must be a non-compressed image
    {
        return(-3);
    }
#endif

    pUC = (unsigned char *)&pInfo->img_height;
    *fh = (*(pUC+1) << 8) | *pUC;

    pUC = (unsigned char *)&pInfo->img_width;
    *fw = (*(pUC+1) << 8) | *pUC;

    // BMP header info is 54 bytes, which is not 32-bit aligned. This causes 
    // access problems for DMAC so we must copy from non-word-aligned address 
    // to a word-aligned address
    for(i = 0; i < 3**fh**fw; i++)
    {
        pBMPImg[i] = pBMP[i];
    }

    return 0;
}
