/*==================================================================================================

    Module Name:  ihapp.c

    General Description: IPU test appl.

====================================================================================================
Copyright (c) 2006-2007 Motorola, Inc.

Revision History:
                            Modification     Tracking
Author (core ID)                Date          Number     Description of Changes
-------------------------   ------------    ----------   -------------------------------------------
Motorola                    Feb.14.2006                  Creation.
Motorola                    Jul.03.2007                  Compliance issue fix

Portability: Indicate if this module is portable to other compilers or
  platforms. If not, indicate specific reasons why is it not portable.


====================================================================================================
                                        INCLUDE FILES
==================================================================================================*/

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#include "ih.h"

#include "fruits_240_320.h"
#include "Beach240_320.h"
#include "qvga_gomango.h"
#include "YUV420.h"

//#ifdef WHEN_I2C_IS_READY
//#include "i2c.h"
//#include "mt9m111.h"
//#endif

static unsigned short *pDispBuf16; /* display buffer pointer */

static uint32_t disp_h = 320, disp_w = 240;

static struct ih_buf_info buf_info[6];

#ifdef USE_LOCAL_REG_TEST
void ipu_show_reg_base(void);
void ipu_read_reg(uint32_t reg, uint32_t len);
void ipu_write_reg(uint32_t reg, uint32_t val);
void ipu_show_cpm(uint32_t dma_ch_num);
#endif

static void
configure_sdc_panel
(
    void
)
{
    ipu_di_signal_cfg_t sig = 
    {
        .datamask_en = 0,
        .clksel_en   = 0,
        .clkidle_en  = 0,
        .data_pol    = 0,    // true = inverted
        .clk_pol     = 1,    // true = rising edge
        .enable_pol  = 1,
        .Hsync_pol   = 0,    // true = active high
        .Vsync_pol   = 0,
        .rsvd        = 0 
    };
    
    ipu_sdc_init_panel(IPU_PANEL_TFT, 60, disp_w, disp_h, IPU_PIX_FMT_RGB666,
                       30, 10, 10, 4, 2, 2, sig, 1);

    ipu_sdc_set_window_pos(MEM_SDC_BG, 0, 0);
}

/*================================================================================================*/

#ifdef WHEN_I2C_IS_READY
static void i2c_test(void)
{
    int addr = 0x6c;
    unsigned short i2cVal;

    // How to manip GPIOs?

scanf("%X", &addr); getchar();
    I2C_open(0x38); // Set I2C_CLK to 100KHZ (Source = ipg_clk) 64MHz/640 = 100KHz

#if 0
    // Map register page to SENS CORE 
    I2C_write(MICRON_SLAVE_ADDR, MT9M111_REG_PAGE_MAP, MT9M111_PAGE_MAP_SENS_CORE);

    // soft reset
    I2C_write(MICRON_SLAVE_ADDR, MT9M111_SC_REG_RESET, 1);
    usleep(200);
    I2C_write(MICRON_SLAVE_ADDR, MT9M111_SC_REG_RESET, 0);

    I2C_read(MICRON_SLAVE_ADDR, 0, &i2cVal);
    printf("Micron MT9M111 chip Id:%X\n", i2cVal);
#endif
printf("about to write to backlight to addr: %X\n", addr); getchar();
    /* backlight on */
    printf("write result: %d\n", I2C_write(addr, 0x10, 0x300));
    I2C_write(addr, 0xA0, 0xFF00);
}
#endif

/*================================================================================================*/

#ifdef USE_LOCAL_REG_TEST
static void reg_read(void)
{
    unsigned int regBaseStart, len;

    printf("Enter: (0x)reg base start addr and length(dec)\n");
    scanf("%08X %i", &regBaseStart, &len);

    ipu_read_reg(regBaseStart, len);
}

static void dump_cpm(void)
{
    unsigned int dma;

    printf("Enter: DMA Ch Num\n");
    scanf("%i", &dma);

    ipu_show_cpm(dma);
}

static void reg_write(void)
{
    unsigned int regBaseStart, len;

    printf("Enter: (0x)reg base off (0..0x1bc) and value(0x)\n");
    scanf("%08X %08X", &regBaseStart, &len);

    ipu_write_reg(regBaseStart, len);
}

static void reg_base(void)
{
    ipu_show_reg_base();
}
#endif
/*================================================================================================*/

static void alloc_disp_buf(void)
{
    buf_info[0].size = disp_h*disp_w*3+1024;
    ipu_alloc_channel_buffer(&buf_info[0]);

    buf_info[1].size = disp_h*disp_w*3+128;
    ipu_alloc_channel_buffer(&buf_info[1]);

    buf_info[2].size = disp_h*disp_w*3+128;
    ipu_alloc_channel_buffer(&buf_info[2]);

    buf_info[3].size = 176*144*3; //YUVQ input buffer
    ipu_alloc_channel_buffer(&buf_info[3]);

    buf_info[4].size = 176*144*3; //YUVQ output buffer
    ipu_alloc_channel_buffer(&buf_info[4]);
}

/*================================================================================================*/

static void free_disp_buf(void)
{
    ipu_free_channel_buffer(&buf_info[0]);

    ipu_free_channel_buffer(&buf_info[1]);

    ipu_free_channel_buffer(&buf_info[2]);

    ipu_free_channel_buffer(&buf_info[3]);

    ipu_free_channel_buffer(&buf_info[4]);
}

/*================================================================================================*/

static void displayBMP(void)
{
    FILE *fptr;
    int rc;
    unsigned int img_fh, img_fw, numBytes;
    ipu_channel_fmt_t chan_fmt;

    while(ipu_is_channel_busy(MEM_SDC_BG));
    ipu_disable_channel(MEM_SDC_BG);
    ipu_uninit_channel(MEM_SDC_BG);

    if((fptr = fopen("qvga_img.bmp", "rb")) == NULL)
    {
        printf("Could not open BMP file\n");
        return;
    }

    numBytes = disp_h * disp_w * 3 + 64; // QVGA, 24 bit, BMP hdr

    fread((void *)buf_info[0].virt_addr, sizeof( char ), numBytes, fptr);

    fclose(fptr);

    if((rc = bmp_decode((unsigned char *)buf_info[0].virt_addr, (unsigned char *)buf_info[1].virt_addr, &img_fh, &img_fw)) != 0)
    {
        printf("Could not decode BMP file (%d)\n", rc);
        return;
    }

    printf("Decoded BMP file: %dx%d\n", img_fw, img_fh);

    chan_fmt.width  = img_fw;
    chan_fmt.height = img_fh;
    chan_fmt.pixel_fmt = IPU_PIX_FMT_BGR24;

    ipu_init_channel(MEM_SDC_BG, NULL);
    
    if(ipu_init_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER_ROTATE_VERT_FLIP,
                               &chan_fmt, chan_fmt.width,
                               (void *)buf_info[1].phys_addr, NULL) != 0)
    {
        printf("Could not initialize DMA buffer\n");
        ipu_uninit_channel(MEM_SDC_BG);
        return;
    }

    ipu_enable_channel(MEM_SDC_BG);
    ipu_select_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0);
}

/*================================================================================================*/

static void simple_test(void)
{
    int i, num_lines;
    ipu_channel_fmt_t chan_fmt;

    while(ipu_is_channel_busy(MEM_SDC_BG));
    ipu_disable_channel(MEM_SDC_BG);
    ipu_uninit_channel(MEM_SDC_BG);

    num_lines = disp_h / 6;
    num_lines -= num_lines % 10;

    pDispBuf16 = (unsigned short *)buf_info[0].virt_addr;
    for(i=0; i<disp_w*num_lines*1; i++) // R
        *pDispBuf16++ = 0xF800;
    for(; i<disp_w*num_lines*2; i++) // Y
        *pDispBuf16++ = 0xFFE0;
    for(; i<disp_w*num_lines*3; i++) // G
        *pDispBuf16++ = 0x07E0;
    for(; i<disp_w*num_lines*4; i++) // W
        *pDispBuf16++ = 0xFFFF;
    for(; i<disp_w*num_lines*5; i++) // C
        *pDispBuf16++ = 0x07FF;
    for(; i<disp_w*num_lines*6; i++) // B
        *pDispBuf16++ = 0x001F;
    for(; i<disp_w*disp_h; i++) 
        *pDispBuf16++ = 0xF38C;

    chan_fmt.width  = disp_w;
    chan_fmt.height = disp_h;
    chan_fmt.pixel_fmt = IPU_PIX_FMT_RGB565;

    ipu_init_channel(MEM_SDC_BG, NULL);
    ipu_init_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[0].phys_addr, NULL);

    ipu_enable_channel(MEM_SDC_BG);
    ipu_select_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0);

    sleep(1);

    memcpy((void *)buf_info[1].virt_addr, (void *)fruits_240_320, disp_h*disp_w*2);
    ipu_update_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0,
                              (void *)buf_info[1].phys_addr);

    printf("hit <cr> to see beach\n");    getchar();

    memcpy((void *)buf_info[2].virt_addr, (void *)Beach240_320, disp_h*disp_w*2);
    ipu_update_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0,
                              (void *)buf_info[2].phys_addr);

    printf("hit <cr> to scribble into disp buf\n");    getchar();
    
    pDispBuf16 = (unsigned short *)buf_info[2].virt_addr;
    for(i=0; i<disp_w*num_lines*1; i++)
        *pDispBuf16++; // move pointer towards middle of display

    for(; i<disp_w*num_lines*2; i++) 
        *pDispBuf16++ = 0xFFE0;
}

/*================================================================================================*/

static void FG_BG_test(void)
{
    // 196 * 240 FG (fruits)
    int i;
    ipu_channel_fmt_t chan_fmt;
    ipu_channel_params_t ch_params;

    pDispBuf16 = (unsigned short *)buf_info[1].virt_addr;
    for(i=0; i<240*65; i++) // R
        *pDispBuf16++ = 0xF800;
    for(i=0; i<240*66; i++) // G
        *pDispBuf16++ = 0x07E0;
    for(i=0; i<240*65; i++) // B
        *pDispBuf16++ = 0x001F;
    pDispBuf16 = (unsigned short *)buf_info[0].virt_addr;
    for(i=0; i<240*196; i++) // Color key
        *pDispBuf16++ = 0x1801;
    for(i=0; i<240*24; i++) // W
        *pDispBuf16++ = 0xFFFF;
    for(i=0; i<240*50; i++) // C
        *pDispBuf16++ = 0x07FF;
    for(i=0; i<240*50; i++) // B
        *pDispBuf16++ = 0x001F;


    while(ipu_is_channel_busy(MEM_SDC_BG));
    ipu_disable_channel(MEM_SDC_BG);
    ipu_uninit_channel(MEM_SDC_BG);

    while(ipu_is_channel_busy(MEM_SDC_FG));
    ipu_disable_channel(MEM_SDC_FG);
    ipu_uninit_channel(MEM_SDC_FG);

    ipu_sdc_set_window_pos(MEM_SDC_FG, 0, 0);
    ipu_sdc_set_color_key(MEM_SDC_BG, 1, 0x180008);
    ipu_sdc_set_global_alpha(1, 0xFF);

    ipu_init_channel(MEM_SDC_BG, NULL);
    ipu_init_channel(MEM_SDC_FG, NULL);

    chan_fmt.height = disp_h;
    chan_fmt.width  = disp_w;
    chan_fmt.pixel_fmt = IPU_PIX_FMT_RGB565;

    ipu_init_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, &chan_fmt,
                            chan_fmt.width, (void *)buf_info[0].phys_addr, NULL);
    chan_fmt.height = 196;
    ipu_init_channel_buffer(MEM_SDC_FG, IPU_INPUT_BUFFER, &chan_fmt,
                            chan_fmt.width, (void *)buf_info[1].phys_addr, NULL);

    ipu_enable_channel(MEM_SDC_BG);
    ipu_enable_channel(MEM_SDC_FG);
    ipu_select_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_SDC_FG, IPU_INPUT_BUFFER, 0);

    printf("hit <cr> to continue\n"); getchar();

    ch_params.in.width  = disp_w;
    ch_params.in.height = disp_h;
    ch_params.in.pixel_fmt = IPU_PIX_FMT_RGB565;
    ch_params.out.width  = disp_w;
    ch_params.out.height = 196;
    ch_params.out.pixel_fmt = IPU_PIX_FMT_RGB565;
    ch_params.specific.mem_pp_mem.graphics_combine_en = 0;

    ipu_init_channel(MEM_PP_MEM, &ch_params);

    chan_fmt.height = disp_h;
    memcpy((void *)buf_info[2].virt_addr, (void *)fruits_240_320, disp_h*disp_w*2);
    ipu_init_channel_buffer(MEM_PP_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[2].phys_addr, NULL);

    chan_fmt.height = 196;
    ipu_init_channel_buffer(MEM_PP_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[1].phys_addr, NULL);

    ipu_enable_channel(MEM_PP_MEM);
    ipu_select_buffer(MEM_PP_MEM, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PP_MEM, IPU_OUTPUT_BUFFER, 0);

    printf("hit <cr> to deinit\n"); getchar();

    ipu_sdc_set_color_key(MEM_SDC_BG, 0, 0);
    ipu_sdc_set_global_alpha(0, 0);

    ipu_disable_channel(MEM_PP_MEM);
    ipu_disable_channel(MEM_SDC_FG);
    ipu_uninit_channel(MEM_PP_MEM);
    ipu_uninit_channel(MEM_SDC_FG);
}

/*================================================================================================*/

static void csc_test(void)
{
    // Post filter MPEG4 QCIF clip. Convert YUV to RGB. Rotate 90. Resize to QVGA. 
    // cannot resize along with the CSC b/c we have to rotate!! Rotating forces us to resize afterwards
    ipu_channel_fmt_t chan_fmt;
    ipu_channel_params_t chan_params;
    int fh = 144, fw = 176, y_uv_ratio = 4, mb_ratio = 16;

    while(ipu_is_channel_busy(MEM_SDC_BG));
    ipu_disable_channel(MEM_SDC_BG);
    ipu_uninit_channel(MEM_SDC_BG);

    memcpy((void *)buf_info[3].virt_addr, (void *)yuv420, sizeof(yuv420));
    /*
      Configure for Post Filter.
      Note that all channels (i.e. QP,Y,U,V) must be specified to 
      use a generic pixel format.
    */
    chan_params.specific.mem_pf_mem.operation = PF_MPEG4_DEBLOCK_DERING;
    ipu_init_channel(MEM_PF_Y_MEM, &chan_params);

    chan_fmt.width  = fw/mb_ratio;
    chan_fmt.height = fh/mb_ratio;
    chan_fmt.pixel_fmt = IPU_PIX_FMT_GENERIC;//IPU_PIX_FMT_YUV420P;

    ipu_init_channel_buffer(MEM_PF_Y_MEM, IPU_SEC_INPUT_BUFFER,
                            &chan_fmt, (chan_fmt.width+3) & ~3,
                            (void *)buf_info[3].phys_addr+fw*fh+fw*fh/y_uv_ratio+fw*fh/y_uv_ratio, NULL);

    chan_fmt.width  = fw;
    chan_fmt.height = fh;
    ipu_init_channel_buffer(MEM_PF_Y_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[3].phys_addr, NULL);

    ipu_init_channel_buffer(MEM_PF_U_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[3].phys_addr+fw*fh, NULL);

    ipu_init_channel_buffer(MEM_PF_V_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[3].phys_addr+fw*fh+fw*fh/y_uv_ratio, NULL);

    ipu_init_channel_buffer(MEM_PF_Y_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[4].phys_addr, NULL);

    ipu_init_channel_buffer(MEM_PF_U_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[4].phys_addr+fw*fh, NULL);

    ipu_init_channel_buffer(MEM_PF_V_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[4].phys_addr+fw*fh+fw*fh/y_uv_ratio, NULL);

    ipu_enable_channel(MEM_PF_Y_MEM);
    ipu_enable_channel(MEM_PF_U_MEM);
    ipu_enable_channel(MEM_PF_V_MEM);

    ipu_select_buffer(MEM_PF_Y_MEM, IPU_OUTPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PF_U_MEM, IPU_OUTPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PF_V_MEM, IPU_OUTPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PF_U_MEM, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PF_V_MEM, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PF_Y_MEM, IPU_SEC_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PF_Y_MEM, IPU_INPUT_BUFFER, 0);

    while(ipu_is_channel_busy(MEM_PF_Y_MEM)); // Post filter
    while(ipu_is_channel_busy(MEM_PF_U_MEM)); // Post filter
    while(ipu_is_channel_busy(MEM_PF_V_MEM)); // Post filter

    ipu_disable_channel(MEM_PF_V_MEM);
    ipu_disable_channel(MEM_PF_U_MEM);
    ipu_disable_channel(MEM_PF_Y_MEM);
    ipu_uninit_channel(MEM_PF_Y_MEM);

    /*
      Now configure for CSC
    */
    chan_fmt.pixel_fmt = IPU_PIX_FMT_YUV420P;
    chan_params.in.width = 
        chan_params.out.width = chan_fmt.width;

    chan_params.in.height = 
        chan_params.out.height = chan_fmt.height;

    chan_params.in.pixel_fmt  = chan_fmt.pixel_fmt;
    chan_params.out.pixel_fmt = IPU_PIX_FMT_RGB565;

    chan_params.specific.mem_pp_mem.graphics_combine_en = 0;

    ipu_init_channel(MEM_PP_MEM, &chan_params);

    ipu_init_channel_buffer(MEM_PP_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[4].phys_addr, NULL);

    chan_fmt.pixel_fmt = IPU_PIX_FMT_RGB565;
    ipu_init_channel_buffer(MEM_PP_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[0].phys_addr, NULL);

    ipu_enable_channel(MEM_PP_MEM);

    ipu_select_buffer(MEM_PP_MEM, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PP_MEM, IPU_OUTPUT_BUFFER, 0);

    while(ipu_is_channel_busy(MEM_PP_MEM)); // CSC (YUV->RGB)

    ipu_disable_channel(MEM_PP_MEM);
    ipu_uninit_channel(MEM_PP_MEM);

    /*
      Now configure for rotate
    */
    ipu_init_channel(MEM_ROT_PP_MEM, NULL);

    ipu_init_channel_buffer(MEM_ROT_PP_MEM, IPU_INPUT_BUFFER_ROTATE_90_RIGHT,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[0].phys_addr, NULL);

    chan_fmt.width  = fh;
    chan_fmt.height = fw;
    ipu_init_channel_buffer(MEM_ROT_PP_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[2].phys_addr, NULL);

    //causes is_chan_busy() to hang                ipu_link_channels(MEM_ROT_PP_MEM, MEM_SDC_BG);
    ipu_enable_channel(MEM_ROT_PP_MEM);

    ipu_select_buffer(MEM_ROT_PP_MEM, IPU_INPUT_BUFFER_ROTATE_90_RIGHT, 0);
    ipu_select_buffer(MEM_ROT_PP_MEM, IPU_OUTPUT_BUFFER, 0);

    while(ipu_is_channel_busy(MEM_ROT_PP_MEM)); // Rotate

    ipu_disable_channel(MEM_ROT_PP_MEM);
    ipu_uninit_channel(MEM_ROT_PP_MEM);

    /*
      Now configure to resize to QVGA
    */
    fh = 320; fw = 240;
    chan_params.in.width  = chan_fmt.width;
    chan_params.out.width = fw;

    chan_params.in.height  = chan_fmt.height;
    chan_params.out.height = fh;

    chan_params.in.pixel_fmt  = 
        chan_params.out.pixel_fmt = chan_fmt.pixel_fmt;

    chan_params.specific.mem_pp_mem.graphics_combine_en = 0;

    ipu_init_channel(MEM_PP_MEM, &chan_params);

    ipu_init_channel_buffer(MEM_PP_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[2].phys_addr, NULL);

    chan_fmt.width  = fw;
    chan_fmt.height = fh;
    ipu_init_channel_buffer(MEM_PP_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[1].phys_addr, NULL);

    ipu_enable_channel(MEM_PP_MEM);

    ipu_select_buffer(MEM_PP_MEM, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_PP_MEM, IPU_OUTPUT_BUFFER, 0);

    while(ipu_is_channel_busy(MEM_PP_MEM)); // Resize

    ipu_disable_channel(MEM_PP_MEM);
    ipu_uninit_channel(MEM_PP_MEM);

    ipu_init_channel(MEM_SDC_BG, NULL);

    ipu_init_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[1].phys_addr, NULL);

    ipu_enable_channel(MEM_SDC_BG);
    ipu_select_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0);
}

/*================================================================================================*/

static void pipVFautoBltRU(void)
{
    int i,j;
    ipu_channel_fmt_t chan_fmt;

    while(ipu_is_channel_busy(MEM_SDC_BG));
    ipu_disable_channel(MEM_SDC_BG);
    ipu_uninit_channel(MEM_SDC_BG);

    while(ipu_is_channel_busy(MEM_SDC_FG));
    ipu_disable_channel(MEM_SDC_FG);
    ipu_uninit_channel(MEM_SDC_FG);

    pDispBuf16 = (unsigned short *)buf_info[2].virt_addr;
    for(i=0; i<72; i++)
    {
        for(j=0; j<30; j++) // R
            *pDispBuf16++ = 0xF800;
        for(j=0; j<30; j++) // G
            *pDispBuf16++ = 0x07E0;
        for(j=0; j<28; j++) // B
            *pDispBuf16++ = 0x001F;
    }
    pDispBuf16 = (unsigned short *)buf_info[3].virt_addr;
    for(i=0; i<88*25; i++) // R
       *pDispBuf16++ = 0xF800;
    for(i=0; i<88*25; i++) // G
        *pDispBuf16++ = 0x07E0;
    for(i=0; i<88*22; i++) // B
        *pDispBuf16++ = 0x001F;
    pDispBuf16 = (unsigned short *)buf_info[1].virt_addr;
    for(i=0; i<240*65; i++) // R
        *pDispBuf16++ = 0xF800;
    for(i=0; i<240*66; i++) // G
        *pDispBuf16++ = 0x07E0;
    for(i=0; i<240*65; i++) // B
        *pDispBuf16++ = 0x001F;
    pDispBuf16 = (unsigned short *)buf_info[0].virt_addr;
    for(i=0; i<240*196; i++) // Color key
        *pDispBuf16++ = 0x1801;
    for(i=0; i<240*24; i++) // W
        *pDispBuf16++ = 0xFFFF;
    for(i=0; i<240*50; i++) // C
        *pDispBuf16++ = 0x07FF;
    for(i=0; i<240*50; i++) // B
        *pDispBuf16++ = 0x001F;

    ipu_sdc_set_window_pos(MEM_SDC_FG, 0, 0);
    ipu_sdc_set_color_key(MEM_SDC_BG, 1, 0x180008);
    ipu_sdc_set_global_alpha(1, 0xFF);

    ipu_init_channel(MEM_SDC_BG, NULL);
    ipu_init_channel(MEM_SDC_FG, NULL);

    chan_fmt.height = disp_h;
    chan_fmt.width  = disp_w;
    chan_fmt.pixel_fmt = IPU_PIX_FMT_RGB565;

    ipu_init_channel_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, &chan_fmt,
                            chan_fmt.width, (void *)buf_info[0].phys_addr, NULL);
    chan_fmt.height = 196;
    ipu_init_channel_buffer(MEM_SDC_FG, IPU_INPUT_BUFFER, &chan_fmt,
                            chan_fmt.width, (void *)buf_info[1].phys_addr, NULL);

    ipu_enable_channel(MEM_SDC_BG);
    ipu_enable_channel(MEM_SDC_FG);
    ipu_select_buffer(MEM_SDC_BG, IPU_INPUT_BUFFER, 0);
    ipu_select_buffer(MEM_SDC_FG, IPU_INPUT_BUFFER, 0);

    /*
        Use the Rotation Unit to BLiT a PIP image into the BG display buffer.
    */
    chan_fmt.width  = 88;
    chan_fmt.height = 72;

    ipu_init_channel(MEM_ROT_PP_MEM, NULL);

    ipu_init_channel_buffer(MEM_ROT_PP_MEM, IPU_INPUT_BUFFER,
                            &chan_fmt, chan_fmt.width,
                            (void *)buf_info[2].phys_addr, (void *)buf_info[3].phys_addr);

    ipu_init_channel_buffer(MEM_ROT_PP_MEM, IPU_OUTPUT_BUFFER,
                            &chan_fmt, disp_w,
                            (void *)buf_info[0].phys_addr+182*240*2, NULL);

    ipu_enable_channel(MEM_ROT_PP_MEM);

    /*
        Register the IRQ handlers needed to perform the VT state machine.
    */
    ipu_register_int_handler(IPU_IRQ_SDC_FG_OUT_EOF, 0);// IPU_IRQ_SDC_FG_OUT_EOF_VT_BLT_HDLR);
    ipu_register_int_handler(IPU_IRQ_SDC_BG_OUT_EOF, 1);// IPU_IRQ_SDC_BG_OUT_EOF_VT_BLT_HDLR);
    ipu_register_int_handler(IPU_IRQ_PP_ROT_IN_EOF, 2); // IPU_IRQ_PP_ROT_IN_EOF_VT_BLT_HDLR);

    ipu_enable_int(IPU_IRQ_SDC_FG_OUT_EOF); // start the VT state machine

    ipu_wait_for_int_event(0); // receive notification from IPU_IRQ_SDC_FG_OUT_EOF_VT_BLT_HDLR

    ipu_disable_int(IPU_IRQ_SDC_FG_OUT_EOF); // terminate the VT state machine

    ipu_register_int_handler(IPU_IRQ_SDC_FG_OUT_EOF, 3);// IPU_IRQ_HDLR_NULL);
    ipu_register_int_handler(IPU_IRQ_SDC_BG_OUT_EOF, 3);
    ipu_register_int_handler(IPU_IRQ_PP_ROT_IN_EOF, 3);

    ipu_disable_channel(MEM_ROT_PP_MEM);
    ipu_disable_channel(MEM_SDC_BG);
    ipu_disable_channel(MEM_SDC_FG);
    ipu_uninit_channel(MEM_ROT_PP_MEM);
    ipu_uninit_channel(MEM_SDC_BG);
    ipu_uninit_channel(MEM_SDC_FG);

    ipu_sdc_set_color_key(MEM_SDC_BG, 0, 0);
    ipu_sdc_set_global_alpha(0, 0);
}

int main (void)
{
    int numTest;

    alloc_disp_buf();
    configure_sdc_panel();
    free_disp_buf();
    alloc_disp_buf();

    while(1)
    {
        printf("1) simple display test\n2) BMP test\n3) i2c test\n5) CSC, resize test\n10)FG/BG test\n20)BLiT test\n\nAny other number to exit\n");
        scanf("%i", &numTest);
        getchar();
        switch(numTest)
        {
            case 1:
                simple_test();
            break;

            case 2:
                displayBMP();
            break;

/*            case 3:
                i2c_test();
            break;
*/

            case 5:
                csc_test();
            break;

            case 10:
                FG_BG_test();
            break;

            case 20:
                pipVFautoBltRU();
            break;
#ifdef USE_LOCAL_REG_TEST
            case 40:
                reg_base();
            break;

            case 50:
                reg_read();
            break;

            case 60:
                reg_write();
            break;

            case 70:
                dump_cpm();
            break;
#endif

            case 80: // gpio test
//                iomux_config_mux(PIN_GPIO6, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE);
  //              gpio_config(0, 6, 1, GPIO_INT_NONE);
                /* Clear the SD/Mode signal */
    //            gpio_set_data(0, 6, 0);
            break;

            default:
                goto exit_main;
        } // switch
    } // while

exit_main:

    free_disp_buf();

    return 0;
}
	
